//===-- CoreFoundationCPP.h -------------------------------------*- C++ -*-===//
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
//===----------------------------------------------------------------------===//
//
//  CoreFoundationCPP.h
//  CoreFoundationCPP
//
//  Created by Greg Clayton on 4/23/09.
//
//

#ifndef CoreFoundationCPP_CoreFoundationCPP_H_
#define CoreFoundationCPP_CoreFoundationCPP_H_

#include <CoreFoundationCPP/CFCBundle.h>
#include <CoreFoundationCPP/CFCData.h>
#include <CoreFoundationCPP/CFCMutableArray.h>
#include <CoreFoundationCPP/CFCMutableDictionary.h>
#include <CoreFoundationCPP/CFCMutableSet.h>
#include <CoreFoundationCPP/CFCReleaser.h>
#include <CoreFoundationCPP/CFCString.h>

#endif // CoreFoundationCPP_CoreFoundationCPP_H_
