#define T unsigned int
#define T_CSTR "unsigned int"
#define T_VALUE_1 11001110
#define T_VALUE_2 22002220
#define T_VALUE_3 33003330
#define T_VALUE_4 44004440
#define T_PRINTF_FORMAT "%u"

#include "basic_type.cpp"
