pre {
  padding: 7px;
}

div.document {
  width: 90%;
}

div.body {
  max-width: 90%;
}

div.note {
  padding: 20px 20px 10px 20px;
}

div.note p.admonition-title {
  font-size: 130%;
}

table.mapping {
  width: 100%;
}

table.mapping td {
  width: 50%;
  padding: 5px;
}

table.mapping td.hed {
  background: #606060;
  color: #fefefe;
  text-align: left;
  border-bottom: 2px #fefefe solid;
  font-weight: bold;
}

table.mapping td.header {
  background: #eeeeee;
}

table.mapping td.content {
  font-family: monospace;
  padding-bottom: 15px;
}

/* Workaround for a Safari bug that would otherwise make table cells less wide
than the containing text. This just sets it back to the default browser
property.*/
td {
  -webkit-hyphens: manual !important;
}

div.sphinxsidebar .caption {
  font-family: Helvetica, Verdana, sans-serif;
  font-size: 10pt;
  font-weight: bold;
  color: #fefefe;
  background: #606060;
  margin-bottom: 0;
  text-transform: uppercase;
  padding: 7px;
}

div.sphinxsidebar a,
div.sphinxsidebar a:hover {
  border: none;
}

div.sphinxsidebar li {
  padding-left: 7px;
  border-bottom: 1px solid #fefefe;
}

div.sphinxsidebar li:hover {
  background: #eeeeee;
}

div.sphinxsidebar ul {
  background: #e9e9e9;
  margin: 0px;
  padding: 0px;
}

div.sphinxsidebar ul a {
  font-family: Helvetica, Verdana, sans-serif;
  font-size: 8pt;
  display: block;
  padding: 5px 0;
  line-height: 14pt;
}

div.sphinxsidebar ul li.toctree-l1 > a {
  font-size: 100%;
}

div.sphinxsidebar h3 {
  /* Hide Navigation. */
  display: none;
}
